@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
+ * Copyright (C) 2010, 2011 NORDUnet A/S
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -68,7 +69,6 @@
 #include <openssl/err.h>
 #include <openssl/md5.h>
 #include "debug.h"
-#include "list.h"
 #include "hash.h"
 #include "util.h"
 #include "hostport.h"
@@ -77,6 +77,9 @@
 #include "tcp.h"
 #include "tls.h"
 #include "dtls.h"
+#if defined(WANT_FTICKS)
+#include "fticks.h"
+#endif
 
 static struct options options;
 static struct list *clconfs, *srvconfs;
@@ -1664,6 +1667,12 @@ void replyh(struct server *server, unsigned char *buf) {
 	}
     }
 
+#if defined(WANT_FTICKS)
+    if (msg->code == RAD_Access_Accept || msg->code == RAD_Access_Reject)
+	if (options.fticks_reporting && from->conf->fticks_viscountry != NULL)
+	    fticks_log(&options, from, msg, rqout);
+#endif
+
     msg->id = (char)rqout->rq->rqid;
     memcpy(msg->auth, rqout->rq->rqauth, 16);
 
@@ -2254,10 +2263,8 @@ int dynamicconfig(struct server *server) {
 
     close(fd[1]);
     pushgconffile(&cf, fdopen(fd[0], "r"), conf->dynamiclookupcommand);
-    ok = getgenericconfig(&cf, NULL,
-			  "Server", CONF_CBK, confserver_cb, (void *)conf,
-			  NULL
-	);
+    ok = getgenericconfig(&cf, NULL, "Server", CONF_CBK, confserver_cb,
+			  (void *) conf, NULL);
     freegconf(&cf);
 
     if (waitpid(pid, &status, 0) < 0) {
@@ -2266,8 +2273,14 @@ int dynamicconfig(struct server *server) {
     }
 
     if (status) {
-	debug(DBG_INFO, "dynamicconfig: command exited with status %d", WEXITSTATUS(status));
-	goto errexit;
+	if (WEXITSTATUS(status) == 10) {
+	    debug(DBG_INFO, "dynamicconfig: command signals empty config");
+	}
+	else {
+	    debug(DBG_INFO, "dynamicconfig: command exited with status %d",
+		  WEXITSTATUS(status));
+	    goto errexit;
+	}
     }
 
     if (ok)
@@ -2417,10 +2430,12 @@ struct modattr *extractmodattr(char *nameval) {
 struct rewrite *getrewrite(char *alt1, char *alt2) {
     struct rewrite *r;
 
-    if ((r = hash_read(rewriteconfs,  alt1, strlen(alt1))))
-	return r;
-    if ((r = hash_read(rewriteconfs,  alt2, strlen(alt2))))
-	return r;
+    if (alt1)
+	if ((r = hash_read(rewriteconfs,  alt1, strlen(alt1))))
+	    return r;
+    if (alt2)
+	if ((r = hash_read(rewriteconfs,  alt2, strlen(alt2))))
+	    return r;
     return NULL;
 }
 
@@ -2657,22 +2672,26 @@ int confclient_cb(struct gconffile **cf, void *arg, char *block, char *opt, char
     memset(conf, 0, sizeof(struct clsrvconf));
     conf->certnamecheck = 1;
 
-    if (!getgenericconfig(cf, block,
-			  "type", CONF_STR, &conftype,
-			  "host", CONF_MSTR, &conf->hostsrc,
-			  "secret", CONF_STR, &conf->secret,
+    if (!getgenericconfig(
+	    cf, block,
+	    "type", CONF_STR, &conftype,
+	    "host", CONF_MSTR, &conf->hostsrc,
+	    "secret", CONF_STR, &conf->secret,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
-			  "tls", CONF_STR, &conf->tls,
-			  "matchcertificateattribute", CONF_STR, &conf->matchcertattr,
-			  "CertificateNameCheck", CONF_BLN, &conf->certnamecheck,
+	    "tls", CONF_STR, &conf->tls,
+	    "matchcertificateattribute", CONF_STR, &conf->matchcertattr,
+	    "CertificateNameCheck", CONF_BLN, &conf->certnamecheck,
 #endif
-			  "DuplicateInterval", CONF_LINT, &dupinterval,
-			  "addTTL", CONF_LINT, &addttl,
-			  "rewrite", CONF_STR, &rewriteinalias,
-			  "rewriteIn", CONF_STR, &conf->confrewritein,
-			  "rewriteOut", CONF_STR, &conf->confrewriteout,
-			  "rewriteattribute", CONF_STR, &conf->confrewriteusername,
-			  NULL
+	    "DuplicateInterval", CONF_LINT, &dupinterval,
+	    "addTTL", CONF_LINT, &addttl,
+	    "rewrite", CONF_STR, &rewriteinalias,
+	    "rewriteIn", CONF_STR, &conf->confrewritein,
+	    "rewriteOut", CONF_STR, &conf->confrewriteout,
+	    "rewriteattribute", CONF_STR, &conf->confrewriteusername,
+#if defined(WANT_FTICKS)
+	    "fticksVISCOUNTRY", CONF_STR, &conf->fticks_viscountry,
+#endif
+	    NULL
 	    ))
 	debugx(1, DBG_ERR, "configuration error");
 
@@ -2787,7 +2806,7 @@ int compileserverconfig(struct clsrvconf *conf, const char *block) {
 	conf->rewriteout = getrewrite(conf->confrewriteout, NULL);
 
     if (!addhostport(&conf->hostports, conf->hostsrc, conf->portsrc, 0)) {
-	debug(DBG_ERR, "error in block %s, failed to parse %s", block, conf->hostsrc);
+	debug(DBG_ERR, "error in block %s, failed to parse %s", block, *conf->hostsrc);
 	return 0;
     }
 
@@ -3004,6 +3023,11 @@ void getmainconfig(const char *configfile) {
     struct gconffile *cfs;
     char **listenargs[RAD_PROTOCOUNT];
     char *sourcearg[RAD_PROTOCOUNT];
+#if defined(WANT_FTICKS)
+    uint8_t *fticks_reporting_str = NULL;
+    uint8_t *fticks_mac_str = NULL;
+    uint8_t *fticks_key_str = NULL;
+#endif
     int i;
 
     cfs = openconfigfile(configfile);
@@ -3027,36 +3051,42 @@ void getmainconfig(const char *configfile) {
     if (!rewriteconfs)
 	debugx(1, DBG_ERR, "malloc failed");
 
-    if (!getgenericconfig(&cfs, NULL,
+    if (!getgenericconfig(
+	    &cfs, NULL,
 #ifdef RADPROT_UDP
-			  "ListenUDP", CONF_MSTR, &listenargs[RAD_UDP],
-			  "SourceUDP", CONF_STR, &sourcearg[RAD_UDP],
+	    "ListenUDP", CONF_MSTR, &listenargs[RAD_UDP],
+	    "SourceUDP", CONF_STR, &sourcearg[RAD_UDP],
 #endif
 #ifdef RADPROT_TCP
-			  "ListenTCP", CONF_MSTR, &listenargs[RAD_TCP],
-			  "SourceTCP", CONF_STR, &sourcearg[RAD_TCP],
+	    "ListenTCP", CONF_MSTR, &listenargs[RAD_TCP],
+	    "SourceTCP", CONF_STR, &sourcearg[RAD_TCP],
 #endif
 #ifdef RADPROT_TLS
-			  "ListenTLS", CONF_MSTR, &listenargs[RAD_TLS],
-			  "SourceTLS", CONF_STR, &sourcearg[RAD_TLS],
+	    "ListenTLS", CONF_MSTR, &listenargs[RAD_TLS],
+	    "SourceTLS", CONF_STR, &sourcearg[RAD_TLS],
 #endif
 #ifdef RADPROT_DTLS
-			  "ListenDTLS", CONF_MSTR, &listenargs[RAD_DTLS],
-			  "SourceDTLS", CONF_STR, &sourcearg[RAD_DTLS],
+	    "ListenDTLS", CONF_MSTR, &listenargs[RAD_DTLS],
+	    "SourceDTLS", CONF_STR, &sourcearg[RAD_DTLS],
 #endif
-			  "TTLAttribute", CONF_STR, &options.ttlattr,
-			  "addTTL", CONF_LINT, &addttl,
-			  "LogLevel", CONF_LINT, &loglevel,
-			  "LogDestination", CONF_STR, &options.logdestination,
-			  "LoopPrevention", CONF_BLN, &options.loopprevention,
-			  "Client", CONF_CBK, confclient_cb, NULL,
-			  "Server", CONF_CBK, confserver_cb, NULL,
-			  "Realm", CONF_CBK, confrealm_cb, NULL,
+	    "TTLAttribute", CONF_STR, &options.ttlattr,
+	    "addTTL", CONF_LINT, &addttl,
+	    "LogLevel", CONF_LINT, &loglevel,
+	    "LogDestination", CONF_STR, &options.logdestination,
+	    "LoopPrevention", CONF_BLN, &options.loopprevention,
+	    "Client", CONF_CBK, confclient_cb, NULL,
+	    "Server", CONF_CBK, confserver_cb, NULL,
+	    "Realm", CONF_CBK, confrealm_cb, NULL,
 #if defined(RADPROT_TLS) || defined(RADPROT_DTLS)
-			  "TLS", CONF_CBK, conftls_cb, NULL,
+	    "TLS", CONF_CBK, conftls_cb, NULL,
 #endif
-			  "Rewrite", CONF_CBK, confrewrite_cb, NULL,
-			  NULL
+	    "Rewrite", CONF_CBK, confrewrite_cb, NULL,
+#if defined(WANT_FTICKS)
+	    "FTicksReporting", CONF_STR, &fticks_reporting_str,
+	    "FTicksMAC", CONF_STR, &fticks_mac_str,
+	    "FTicksKey", CONF_STR, &fticks_key_str,
+#endif
+	    NULL
 	    ))
 	debugx(1, DBG_ERR, "configuration error");
 
@@ -3073,6 +3103,11 @@ void getmainconfig(const char *configfile) {
     if (!setttlattr(&options, DEFAULT_TTL_ATTR))
     	debugx(1, DBG_ERR, "Failed to set TTLAttribute, exiting");
 
+#if defined(WANT_FTICKS)
+    fticks_configure(&options, &fticks_reporting_str, &fticks_mac_str,
+		     &fticks_key_str);
+#endif
+
     for (i = 0; i < RAD_PROTOCOUNT; i++)
 	if (listenargs[i] || sourcearg[i])
 	    setprotoopts(i, listenargs[i], sourcearg[i]);
@@ -3178,7 +3213,7 @@ int createpidfile(const char *pidfile) {
     return f && !fclose(f) && r >= 0;
 }
 
-int main(int argc, char **argv) {
+int radsecproxy_main(int argc, char **argv) {
     pthread_t sigth;
     sigset_t sigset;
     struct list_node *entry;
