@@ -1,11 +1,17 @@
 /*
  * Copyright (C) 2006-2009 Stig Venaas <venaas@uninett.no>
+ * Copyright (C) 2010,2011 NORDUnet A/S
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  */
 
+#include <sys/time.h>
+#include <stdint.h>
+#include <pthread.h>
+#include <regex.h>
+#include "list.h"
 #include "tlv11.h"
 #include "radmsg.h"
 #include "gconfig.h"
@@ -32,6 +38,21 @@
 #define RAD_DTLS 3
 #define RAD_PROTOCOUNT 4
 
+enum rsp_fticks_reporting_type {
+    RSP_FTICKS_REPORTING_NONE = 0, /* Default.  */
+    RSP_FTICKS_REPORTING_BASIC,
+    RSP_FTICKS_REPORTING_FULL
+};
+
+enum rsp_fticks_mac_type {
+    RSP_FTICKS_MAC_STATIC = 0,
+    RSP_FTICKS_MAC_ORIGINAL,
+    RSP_FTICKS_MAC_VENDOR_HASHED,
+    RSP_FTICKS_MAC_VENDOR_KEY_HASHED, /* Default.  */
+    RSP_FTICKS_MAC_FULLY_HASHED,
+    RSP_FTICKS_MAC_FULLY_KEY_HASHED
+};
+
 struct options {
     char *logdestination;
     char *ttlattr;
@@ -39,6 +60,9 @@ struct options {
     uint8_t addttl;
     uint8_t loglevel;
     uint8_t loopprevention;
+    enum rsp_fticks_reporting_type fticks_reporting;
+    enum rsp_fticks_mac_type fticks_mac;
+    uint8_t *fticks_key;
 };
 
 struct commonprotoopts {
@@ -98,12 +122,14 @@ struct clsrvconf {
     uint8_t dupinterval;
     uint8_t certnamecheck;
     uint8_t addttl;
+    uint8_t loopprevention;
     struct rewrite *rewritein;
     struct rewrite *rewriteout;
     pthread_mutex_t *lock; /* only used for updating clients so far */
     struct tls *tlsconf;
     struct list *clients;
     struct server *servers;
+    char *fticks_viscountry;
 };
 
 #include "tlscommon.h"
@@ -209,3 +235,8 @@ void freerq(struct request *rq);
 int radsrv(struct request *rq);
 void replyh(struct server *server, unsigned char *buf);
 struct addrinfo *resolve_hostport_addrinfo(uint8_t type, char *hostport);
+uint8_t *radattr2ascii(struct tlv *attr);
+
+/* Local Variables: */
+/* c-file-style: "stroustrup" */
+/* End: */
